@@ -1,9 +1,10 @@
 package Net::DNS::SEC::RSA;
 
 #
-# $Id: RSA.pm 1667 2018-04-20 10:01:29Z willem $
+# $Id: RSA.pm 1376 2015-07-12 19:16:04Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1667 $)[1];
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1376 $)[1];
 
 
 =head1 NAME
@@ -29,14 +30,14 @@ generation and verification procedures.
 
     $signature = Net::DNS::SEC::RSA->sign( $sigdata, $private );
 
-Generates the wire-format signature from the sigdata octet string
+Generates the wire-format binary signature from the binary sigdata
 and the appropriate private key object.
 
 =head2 verify
 
     $validated = Net::DNS::SEC::RSA->verify( $sigdata, $keyrr, $sigbin );
 
-Verifies the signature over the sigdata octet string using the specified
+Verifies the signature over the binary sigdata using the specified
 public key resource record.
 
 =cut
@@ -44,59 +45,56 @@ public key resource record.
 use strict;
 use integer;
 use warnings;
+use Carp;
 use MIME::Base64;
 
+use Crypt::OpenSSL::Bignum;
+use Crypt::OpenSSL::RSA 0.27;
+
+
 my %RSA = (
-	1  => [sub { Net::DNS::SEC::libcrypto::EVP_md5() }],
-	5  => [sub { Net::DNS::SEC::libcrypto::EVP_sha1() }],
-	7  => [sub { Net::DNS::SEC::libcrypto::EVP_sha1() }],
-	8  => [sub { Net::DNS::SEC::libcrypto::EVP_sha256() }],
-	10 => [sub { Net::DNS::SEC::libcrypto::EVP_sha512() }],
+	1  => 'use_md5_hash',
+	5  => 'use_sha1_hash',
+	7  => 'use_sha1_hash',
+	8  => 'use_sha256_hash',
+	10 => 'use_sha512_hash',
 	);
 
 
 sub sign {
 	my ( $class, $sigdata, $private ) = @_;
 
-	my $algorithm = $private->algorithm;
-	my ($evpmd) = @{$RSA{$algorithm} || []};
-	die 'private key not RSA' unless $evpmd;
+	my $hash = $RSA{$private->algorithm} || die 'private key not RSA';
 
-	my ( $n, $e, $d, $p, $q ) = map decode_base64( $private->$_ ),
-			qw(Modulus PublicExponent PrivateExponent Prime1 Prime2);
+	my @param = map Crypt::OpenSSL::Bignum->new_from_bin( decode_base64( $private->$_ ) ),
+			qw(Modulus PublicExponent PrivateExponent
+			Prime1 Prime2 Exponent1 Exponent2 Coefficient);
+	my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters(@param);
 
-	my $rsa = Net::DNS::SEC::libcrypto::RSA_new();
-	Net::DNS::SEC::libcrypto::RSA_set0_factors( $rsa, $p, $q );
-	Net::DNS::SEC::libcrypto::RSA_set0_key( $rsa, $n, $e, $d );
-
-	my $evpkey = Net::DNS::SEC::libcrypto::EVP_PKEY_new();
-	Net::DNS::SEC::libcrypto::EVP_PKEY_assign_RSA( $evpkey, $rsa );
-
-	Net::DNS::SEC::libcrypto::EVP_sign( $sigdata, $evpkey, &$evpmd );
+	$rsa->use_pkcs1_oaep_padding;
+	$rsa->$hash;
+	$rsa->sign($sigdata);
 }
 
 
 sub verify {
 	my ( $class, $sigdata, $keyrr, $sigbin ) = @_;
 
-	my $algorithm = $keyrr->algorithm;
-	my ($evpmd) = @{$RSA{$algorithm} || []};
-	die 'public key not RSA' unless $evpmd;
-
-	return unless $sigbin;
+	my $hash = $RSA{$keyrr->algorithm} || die 'public key not RSA';
 
 	my $keybin = $keyrr->keybin;				# public key
 	my ( $short, $long ) = unpack( 'Cn', $keybin );		# RFC3110, section 2
 	my $keyfmt = $short ? "x a$short a*" : "x3 a$long a*";
 	my ( $exponent, $modulus ) = unpack( $keyfmt, $keybin );
 
-	my $rsa = Net::DNS::SEC::libcrypto::RSA_new();
-	Net::DNS::SEC::libcrypto::RSA_set0_key( $rsa, $modulus, $exponent, '' );
+	my $bn_modulus	= Crypt::OpenSSL::Bignum->new_from_bin($modulus);
+	my $bn_exponent = Crypt::OpenSSL::Bignum->new_from_bin($exponent);
 
-	my $evpkey = Net::DNS::SEC::libcrypto::EVP_PKEY_new();
-	Net::DNS::SEC::libcrypto::EVP_PKEY_assign_RSA( $evpkey, $rsa );
+	my $rsa = Crypt::OpenSSL::RSA->new_key_from_parameters( $bn_modulus, $bn_exponent );
+	$rsa->use_pkcs1_oaep_padding;
+	$rsa->$hash;
 
-	Net::DNS::SEC::libcrypto::EVP_verify( $sigdata, $sigbin, $evpkey, &$evpmd );
+	$rsa->verify( $sigdata, $sigbin );
 }
 
 
@@ -108,13 +106,15 @@ __END__
 
 =head1 ACKNOWLEDGMENT
 
-Thanks are due to Eric Young and the many developers and 
-contributors to the OpenSSL cryptographic library.
+Andy Vaskys (Network Associates Laboratories) supplied the code for
+handling RSA with SHA1 (Algorithm 5).
+
+The Crypt::OpenSSL::RSA package was created by Ian Robertson.
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2014,2018 Dick Franks.
+Copyright (c)2014 Dick Franks.
 
 All rights reserved.
 
@@ -141,8 +141,8 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<Net::DNS>, L<Net::DNS::SEC>,
-RFC8017, RFC3110,
-L<OpenSSL|http://www.openssl.org/docs>
+L<Crypt::OpenSSL::RSA>,
+RFC2437, RFC3110
 
 =cut
 
